﻿/*
  ColorChange KeePass Plugin
  Copyright (C) 2015-2018 Dominik Reichl <dominik.reichl@t-online.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using KeePass;
using KeePass.Plugins;
using KeePass.UI;

using KeePassLib;

namespace ColorChange
{
	public sealed class ColorChangeExt : Plugin
	{
		private IPluginHost m_host = null;

		public override bool Initialize(IPluginHost host)
		{
			Terminate();

			if(host == null) return false;
			m_host = host;

			m_host.MainWindow.UIStateUpdated += this.OnUIStateUpdated;

			return true;
		}

		public override void Terminate()
		{
			if(m_host == null) return;

			m_host.MainWindow.UIStateUpdated -= this.OnUIStateUpdated;

			m_host = null;
		}

		private void OnUIStateUpdated(object sender, EventArgs e)
		{
			ListView lv = (m_host.MainWindow.Controls.Find(
				"m_lvEntries", true)[0] as ListView);
			if(lv == null) { Debug.Assert(false); return; }

			DateTime dtNow = DateTime.UtcNow;
			DateTime dtSoon = dtNow.AddDays(Program.Config.Application.ExpirySoonDays);

			lv.BeginUpdate();

			foreach(ListViewItem lvi in lv.Items)
			{
				PwListItem li = (lvi.Tag as PwListItem);
				if(li == null) { Debug.Assert(false); continue; }

				PwEntry pe = li.Entry;
				if(pe == null) { Debug.Assert(false); continue; }

				if(pe.Expires)
				{
					if(pe.ExpiryTime <= dtNow)
						lvi.BackColor = Color.Red;
					else if(pe.ExpiryTime <= dtSoon)
					{
						lvi.BackColor = Color.Orange;
						lvi.ImageIndex = (int)PwIcon.Warning;
					}
				}
			}

			lv.EndUpdate();
		}
	}
}
